Index: res/res_timing_timerfd.c
===================================================================
--- res/res_timing_timerfd.c	(wersja 322188)
+++ res/res_timing_timerfd.c	(kopia robocza)
@@ -37,6 +37,7 @@
 #include "asterisk/logger.h"
 #include "asterisk/utils.h"
 #include "asterisk/time.h"
+#include "asterisk/poll-compat.h"
 
 static void *timing_funcs_handle;
 
@@ -159,10 +160,39 @@
 
 static void timerfd_timer_ack(int handle, unsigned int quantity)
 {
-	uint64_t expirations;
+	uint64_t expirations = 0;
 	int read_result = 0;
+	struct timerfd_timer *our_timer, find_helper = {
+		.handle = handle,
+	};
+	int res;
+	struct pollfd pfd;
 
+	if (!(our_timer = ao2_find(timerfd_timers, &find_helper, OBJ_POINTER))) {
+		ast_log(LOG_ERROR, "Couldn't find timer with handle %d\n", handle);
+		return;
+	}
+
 	do {
+		ao2_lock(our_timer);
+		if (our_timer->saved_timer.it_value.tv_sec == 0 && our_timer->saved_timer.it_value.tv_nsec == 0) {
+			ast_log(LOG_WARNING, "Trying to wait on a disarmed timer\n");
+			ao2_unlock(our_timer);
+			break;
+		}
+		ao2_unlock(our_timer);
+
+		pfd.fd = handle;
+		pfd.events = POLLIN | POLLPRI;
+
+		res = ast_poll(&pfd, 1, 1000);
+		if (res == 0) {
+			ast_log(LOG_WARNING, "Timerfd timer has not expired for 1 second\n");
+			continue;
+		} else if (res != 1 && (errno != EAGAIN && errno != EINTR)) {
+			ast_log(LOG_ERROR, "Read error: %s\n", strerror(errno));
+			break;
+		}
 		read_result = read(handle, &expirations, sizeof(expirations));
 		if (read_result == -1) {
 			if (errno == EINTR || errno == EAGAIN) {
@@ -174,6 +204,8 @@
 		}
 	} while (read_result != sizeof(expirations));
 
+	ao2_ref(our_timer, -1);
+
 	if (expirations != quantity) {
 		ast_debug(2, "Expected to acknowledge %u ticks but got %llu instead\n", quantity, (unsigned long long) expirations);
 	}
